@@ -55,17 +55,18 @@ using namespace std;
 extern int trace_on;
 
 int main2();
+int main3(bool waitForGdbConnection);
 int main4();
 
 int main(int argc, char *argv[]) {
     int c;
-    //int digit_optind = 0;
     bool gdbserver_flag=0;
     string filename("unknown");
     string devicename("unknown");
     string tracefilename("unknown");
     int global_gdbserver_port    = 1212;
     int global_gdb_debug         = 0;
+    bool globalWaitForGdbConnection=true; //please wait for gdb connection
     int extended_tests=0;
     int userinterface_flag=0;
     UserInterface *ui;
@@ -82,10 +83,11 @@ int main(int argc, char *argv[]) {
             {"gdbserver", 1, 0, 'g'},
             {"trace", 1, 0, 't'},
             {"version",0,0,'v'},
+            {"nogdbwait",0,0,'n'},
             {0, 0, 0, 0}
         };
 
-        c = getopt_long (argc, argv, "f:d:gGd:p:t:uxzhv", long_options, &option_index);
+        c = getopt_long (argc, argv, "f:d:gGd:p:t:uxyzhvn", long_options, &option_index);
         if (c == -1)
             break;
 
@@ -113,6 +115,7 @@ int main(int argc, char *argv[]) {
             case 'G':
                 cout << "Running with debug informations from gdbserver" << endl;
                 global_gdb_debug = 1;
+                gdbserver_flag=1;
                 break;
 
             case 'p':
@@ -130,6 +133,11 @@ int main(int argc, char *argv[]) {
                 extended_tests=1;
                 break;
 
+            case 'y':
+                cout << "Running in other main loop for tests only! DO NOT USE" << endl;
+                extended_tests=2;
+                break;
+
             case 'z':
                 cout << "Running in other main loop for tests only! DO NOT USE" << endl;
                 extended_tests=3;
@@ -137,21 +145,31 @@ int main(int argc, char *argv[]) {
 
             case 'v':
                 {
-                    std::cout << "Simulavr++ " << VERSION << std::endl;
-                    std::cout << "See documentation for copyright and distribution terms" << std::endl;
-                    std::cout << std::endl;
+                    cout << "Simulavr++ " << VERSION << endl;
+                    cout << "See documentation for copyright and distribution terms" << endl;
+                    cout << endl;
                     exit(0);
                 }
                 break;
 
+            case 'n':
+                { 
+                    cout << "We will NOT wait for a gdb connection, simulation starts now!" << endl;
+                    globalWaitForGdbConnection=false;
+                }
+                break;
+
+
             default:
                 cout << "AVR-Simulator" << endl;
                 cout << "-u                           run with user interface for external pin handling at port 7777" << endl;
                 cout << "-f --file <name>             load elf-file <name> for simulation in simulated target" << endl; 
                 cout << "-d --device <device name>    simulate <device name> " << endl;
                 cout << "-g --gdbserver               running as gdb-server" << endl;
+                cout << "-G                           running as gdb-server and write debug info for gdb-connection" << endl;                             
                 cout << "-p  <port>                   change <port> for gdb server to port" << endl;
                 cout << "-t --trace <file name>       enable trace outputs into <file name>" << endl;
+                cout << "-n --nogdbwait               do not wait for gdb connection" << endl;
                 cout << endl;
                 cout << "Supported devices:" << endl;
                 cout << "at90s8515" << endl;
@@ -166,11 +184,14 @@ int main(int argc, char *argv[]) {
 
     if (extended_tests==1) {
         main2();
-        //#warning NOW COMPILED FOR IO TEST ONLY NOT NORMAL -X OPERATION
-        //main3();
         return 0;
     } 
 
+    if (extended_tests ==2) {
+        main3(globalWaitForGdbConnection);
+        return 0;
+    }
+
     if (extended_tests==3) {
         main4();
         return 0;
@@ -191,8 +212,6 @@ int main(int argc, char *argv[]) {
         exit(0);
     }
 
-    dev1->SetClockFreq(250); //4 Mhz for dummy
-    SystemClock::Instance().Add(dev1);
 
 
     if (filename != "unknown" ) {
@@ -202,29 +221,27 @@ int main(int argc, char *argv[]) {
     if (userinterface_flag==1) {
         ui=new UserInterface(7777); //if not gdb, the ui will be master controller :-)
     }
+    
+    dev1->SetClockFreq(250); //4 Mhz for dummy
 
-    if (gdbserver_flag==1) {
-        cout << "Going to gdb..." << endl;
-        gdb_interact( dev1, global_gdbserver_port, global_gdb_debug );
-    } else {
+    if (gdbserver_flag==0) {
         SystemClock::Instance().Add(dev1);
-
         while (1) { //for ever
-            SystemClock::Instance().Step(0);
+            bool untilCoreStepFinished=false;
+            SystemClock::Instance().Step(untilCoreStepFinished);
         }
-
-    }
-
+    } else { //gdb should be activated
+        cout << "Going to gdb..." << endl;
+        GdbServer gdb1(dev1, global_gdbserver_port, global_gdb_debug, globalWaitForGdbConnection);
+        SystemClock::Instance().Add(&gdb1);
+        SystemClock::Instance().Endless();
+    } 
 }
-/*
-void ChildExit(int a) {
-cout << "Child ends.... so we go away, thank you for using me :-)" << endl;
-kill(getpid(), SIGQUIT);
-}
-*/
+
 #define USE_ANA
 #define USE_PWM4
-//#define USE_WEICHE8
+#define USE_WEICHE8
+#define USE_SIGNAL
 #define EXPORT_CLKDATA
 int main2() {
 
@@ -265,7 +282,7 @@ int main2() {
     AvrDevice *dev1= new AvrDevice_at90s8515;
     dev1->Load("pwm4.o.go"); //fahrspannungserzeuger
     //dev1->SetClockFreq(75);    
-    dev1->SetClockFreq(256);    //4Mhz //257 is not working !!! 
+    dev1->SetClockFreq(125);    //4Mhz //257 is not working !!! 256 seems ok
     SystemClock::Instance().Add(dev1);
 
     clk.Add(dev1->GetPin("D2"));  //pwm
@@ -401,6 +418,21 @@ int main2() {
     data.Add(&odWeicheDataW);
     clk.Add(dev5->GetPin("D2")); //weiche
     data.Add(dev5->GetPin("D3")); //weiche data read
+#endif
+
+#ifdef USE_SIGNAL
+    AvrDevice *dev_signal= new AvrDevice_at90s8515;
+    dev_signal->Load("signal.o.go"); //Master
+    dev_signal->SetClockFreq(250);    //250 ns per Cycle -> 4Mhz
+    SystemClock::Instance().Add(dev_signal);
+
+    OpenDrain odSignalDataW( dev_signal->GetPin("D4") ); //weiche data write
+    data.Add(&odSignalDataW);
+    clk.Add(dev_signal->GetPin("D2")); //weiche
+    data.Add(dev_signal->GetPin("D3")); //weiche data read
+
+
+
 #endif
 
     AvrDevice *dev2= new AvrDevice_atmega128;
@@ -593,7 +625,7 @@ int main2() {
 
 int main4() {
 
-    cout << "Starting with gui for main3" << endl;
+    cout << "Starting with gui for main4" << endl;
     UserInterface *ui=new UserInterface(7777);
     SystemClock::Instance().AddAsyncMember(ui);
 
@@ -613,7 +645,8 @@ int main4() {
     SystemClock::Instance().Add(dev1);
     cout << "Device setup complete" << endl;
     while(1) {
-        SystemClock::Instance().Step(0);
+        bool untilCoreStepFinished=false;
+        SystemClock::Instance().Step(untilCoreStepFinished);
     }
 
 
@@ -622,5 +655,3 @@ int main4() {
     return 0;
 }
 
-
-
