@@ -23,9 +23,11 @@
 #ifndef SIMULATIONMEMBER
 #define SIMULATIONMEMBER
 
+#include "systemclocktypes.h"
+
 class SimulationMember {
     public:
-        virtual int Step(bool trueHwStep, unsigned long long *timeToNextStepIn_ns=0)=0;
+        virtual int Step(bool &trueHwStep, SystemClockOffset *timeToNextStepIn_ns=0)=0;
 };
 
 #endif 
